@@ -96,17 +96,22 @@ object_read_bencode(std::istream* input, Object* object, uint32_t depth) {
 
       Object::list_iterator itr = object->as_list().insert(object->as_list().end(), Object());
       object_read_bencode(input, &*itr, depth);
+
+      if (itr->flags() & Object::flag_unordered)
+        object->set_internal_flags(Object::flag_unordered);
     }
 
     break;
 
-  case 'd':
+  case 'd': {
     input->get();
     *object = Object::create_map();
 
     if (++depth >= 1024)
       break;
 
+    Object::string_type last;
+
     while (input->good()) {
       if (input->peek() == 'e') {
 	input->get();
@@ -118,10 +123,20 @@ object_read_bencode(std::istream* input, Object* object, uint32_t depth) {
       if (!object_read_string(input, str))
 	break;
 
-      object_read_bencode(input, &object->as_map()[str], depth);
+      if (last >= str)
+        object->set_internal_flags(Object::flag_unordered);
+
+      Object* value = &object->as_map()[str];
+      object_read_bencode(input, value, depth);
+
+      if (value->flags() & Object::flag_unordered)
+        object->set_internal_flags(Object::flag_unordered);
+
+      str.swap(last);
     }
 
     break;
+  }
 
   default:
     if (c >= '0' && c <= '9') {
@@ -194,6 +209,9 @@ object_write_bencode_c_string(object_write_data_t* output, const char* srcData,
     if (output->pos == output->buffer.second) {
       output->buffer = output->writeFunc(output->data, output->buffer);
       output->pos = output->buffer.first;
+
+      //      if (output->buffer.first == output->buffer.second)
+      //        return;
     }
 
     srcData += len;
@@ -289,6 +307,9 @@ object_write_bencode_c(object_write_t writeFunc, void* data, object_buffer_t buf
 
   object_write_bencode_c_object(&output, object);
 
+  //  if (output->buffer.first == output->buffer.second)
+  //    throw 
+
   // Don't flush the buffer.
   if (output.pos == output.buffer.first)
     return output.buffer;
